package com.dz.openapi.config;


import com.dz.openapi.core.OpenAPIConfig;
import com.dz.openapi.demo.OpenAPIConstants;
import com.dz.openapi.util.RSAUtil;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpRequest;
import org.springframework.http.MediaType;
import org.springframework.http.client.ClientHttpRequestExecution;
import org.springframework.http.client.ClientHttpRequestInterceptor;
import org.springframework.http.client.ClientHttpResponse;
import org.springframework.web.client.RestTemplate;

import java.io.IOException;
import java.util.Objects;

import lombok.extern.slf4j.Slf4j;

@Configuration
@Slf4j
public class APIConfiguration {

    @Bean
    @ConfigurationProperties(prefix = "dz.openapi")
    public OpenAPIConfig openAPIConfig() {
        return new OpenAPIConfig();
    }

    @Bean
    @Autowired
    public RestTemplate openAPITemplate(OpenAPIConfig config) {
        RestTemplate restTemplate = new RestTemplate();
        if (!Objects.isNull(config.getAppId())) {
            restTemplate.getInterceptors().add(new OpenAPIAuthorizationInterceptor(config));
        }
        return restTemplate;
    }


    public class OpenAPIAuthorizationInterceptor implements ClientHttpRequestInterceptor {
        private OpenAPIConfig config;

        public OpenAPIAuthorizationInterceptor(OpenAPIConfig config) {
            this.config = config;
        }

        @Override
        public ClientHttpResponse intercept(HttpRequest request, byte[] body, ClientHttpRequestExecution execution) throws IOException {
            boolean isSign = !request.getHeaders().containsKey(OpenAPIConstants.HEADER_SIGN_SS_NAME);
            HttpHeaders headers = request.getHeaders();

            if (isSign) {
                String timestamp = String.valueOf(System.currentTimeMillis());
                headers.add("appId", config.getAppId());
                headers.add("timestamp", "" + timestamp);
                headers.add("sign", RSAUtil.sign(config.getAppId() + timestamp, config.getPrivateKey()));
                headers.add("Content-Type", MediaType.APPLICATION_JSON_VALUE);
            }

            log.info("request for {}", request.getURI().toString());
            for (String key : headers.keySet()) {
                log.info("header:{}={}", key, headers.get(key));
            }

            return execution.execute(request, body);
        }
    }
}
