package com.dz.openapi.demo.base;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.dz.openapi.core.OpenAPIResponse;
import com.dz.openapi.demo.BaseWebTest;
import com.dz.openapi.demo.OpenAPIConstants;
import com.dz.openapi.util.RSAUtil;
import com.dz.openapi.util.SSORSAUtil;

import org.apache.commons.lang3.StringUtils;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.util.UriComponentsBuilder;

import java.util.HashMap;
import java.util.Map;

import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.extern.slf4j.Slf4j;

@Slf4j
public class TenantSSOTest extends BaseWebTest {

    @Test
    public void ssoTest() throws Exception {
        UserInfo userInfo = new UserInfo();
        userInfo.setThirdpartAccount("9998");
        userInfo.setEmail("9998@msxf.com");
        userInfo.setPhone("99980000000");
        userInfo.setEmpno("9998");
        userInfo.setTimestamp(System.currentTimeMillis());

        ssoLogin(config.getSso().getPublicKey(), userInfo);
//        testlogin();
    }

    private void ssoLogin(String publicKey, UserInfo userInfo) throws Exception {
        HttpHeaders headers = new HttpHeaders();

        // 本地测试使用
        headers.set(OpenAPIConstants.HEADER_SIGN_SS_NAME, "true");
        if (StringUtils.isNotEmpty(config.getDynamicDomain())) {
            headers.set("xhost", config.getDynamicDomain());
        }

        // 登录用户数据加密，publicKey为单点登录页面上的token
        String result = SSORSAUtil.rsaEncrypt(JSON.toJSONString(userInfo), publicKey);

        headers.setContentType(MediaType.APPLICATION_FORM_URLENCODED);

        MultiValueMap<String, String> params = new LinkedMultiValueMap<>();
        params.add("data", result);
        params.add("timestamp", String.valueOf(userInfo.getTimestamp()));

        HttpEntity<MultiValueMap<String, String>> request = new HttpEntity<>(params, headers);
        log.info("--------------SSO请求参数-----------------");
        log.info(JSON.toJSONString(params));

        ResponseEntity<JSONObject> responseEntity = restTemplate.postForEntity(config.getAPIUrl(OpenAPIConstants.Base.POST_TENANT_SSO_URL), request, JSONObject.class);

        log.info("--------------SSO响应结果-----------------");
        assertResultAndLog(responseEntity);
    }

    @Data
    @NoArgsConstructor
    public static class UserInfo {
        private String thirdpartAccount;
        private String email;
        private String phone;
        private String empno;
        private Long timestamp;
    }
}
