package com.dz.openapi.demo.scrm;


import com.alibaba.fastjson.JSON;
import com.dz.openapi.core.OpenAPIResponse;
import com.dz.openapi.demo.BaseWebTest;
import com.dz.openapi.demo.OpenAPIConstants;
import com.dz.openapi.demo.scrm.model.BasicCustomerFieldDefineDTO;
import com.dz.openapi.demo.scrm.model.BasicCustomerFieldMutliCheckbox;
import com.dz.openapi.demo.scrm.model.CustomerDTO;
import com.dz.openapi.demo.scrm.model.CustomerDetailResponse;
import com.dz.openapi.demo.scrm.model.CustomerDictDTO;
import com.dz.openapi.demo.scrm.model.CustomerFieldDefineResponse;

import org.apache.commons.lang3.StringUtils;
import org.junit.jupiter.api.Test;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.util.UriComponentsBuilder;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import lombok.extern.slf4j.Slf4j;

@Slf4j
public class SCRMTest extends BaseWebTest {
    private SimpleDateFormat nameDateFormat = new SimpleDateFormat("yyyyMMddHHmmss");
    private SimpleDateFormat dateFormat = new SimpleDateFormat("yyyyMMddHH:mm:ss");

    @Test
    public void customerTest() {

        // 查询字段定义信息
        log.info("查询字段定义信息");
        CustomerFieldDefineResponse fieldDefineResponse = restTemplate.getForObject(
                config.getAPIUrl(OpenAPIConstants.SCRM.CUSTOMER_FIELD_DEFINE), CustomerFieldDefineResponse.class);
        assertResultAndLog(fieldDefineResponse);

        // 生成demo客户
        log.info("生成demo客户");
        CustomerDTO customerDTO = buildNewCustomer(fieldDefineResponse.getData());

        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        HttpEntity<CustomerDTO> createRequest = new HttpEntity<>(customerDTO, headers);

        // 创建客户
        log.info("创建客户");
        CustomerDetailResponse createResponse = restTemplate.exchange(
                config.getAPIUrl(OpenAPIConstants.SCRM.CUSTOMER_CREATE), HttpMethod.POST, createRequest, CustomerDetailResponse.class).getBody();
        assertResultAndLog(createResponse);

        String customerId = createResponse.getData().getCustomer().getCustomerId();

        // 修改客户
        log.info("修改客户");
        customerDTO.setDesc(customerDTO.getDesc() + " | modified");
        customerDTO.setTag("OpenAPIDemo,ModifyDemo");
        HttpEntity<CustomerDTO> modifyRequest = new HttpEntity<>(customerDTO, headers);
        MultiValueMap<String, String> customerIdMap = new LinkedMultiValueMap<>();
        customerIdMap.add("customerId", customerId);

        UriComponentsBuilder modifyBuilder = UriComponentsBuilder.fromHttpUrl(config.getAPIUrl(OpenAPIConstants.SCRM.CUSTOMER_MODIFY)).queryParams(customerIdMap);
        CustomerDetailResponse modifyResponse = restTemplate.exchange(modifyBuilder.toUriString()
                , HttpMethod.PUT, modifyRequest, CustomerDetailResponse.class).getBody();
        assertResultAndLog(modifyResponse);

        log.info("查询客户 by customerId");
        queryCustomer(customerId, "customerId");

        log.info("查询客户 by externalId");
        queryCustomer(customerDTO.getExternalId(), "externalId");

        log.info("查询客户 by phone");
        queryCustomer(customerDTO.getExternalId(), "phone");

        log.info("分页查询客户");
        customerPageQueryTest("OpenAPIDemo");
    }


    private CustomerDetailResponse queryCustomer(String content, String type) {
        // 查询客户
        MultiValueMap<String, String> queryParamMap = new LinkedMultiValueMap<>();
        queryParamMap.add("content", content);
        queryParamMap.add("type", type);
        UriComponentsBuilder queryBuilder = UriComponentsBuilder.fromHttpUrl(config.getAPIUrl(OpenAPIConstants.SCRM.CUSTOMER_DETAIL)).queryParams(queryParamMap);
        CustomerDetailResponse queryResponse = restTemplate.getForObject(queryBuilder.toUriString(), CustomerDetailResponse.class);
        assertResultAndLog(queryResponse);
        return queryResponse;
    }

    public void customerPageQueryTest(String searchStr) {
        // 分页查询客户
        MultiValueMap<String, String> queryParamMap = new LinkedMultiValueMap<>();
        queryParamMap.add("pageNum", "1");
        queryParamMap.add("pageSize", "10");
        queryParamMap.add("searchStr", searchStr);
        UriComponentsBuilder queryBuilder = UriComponentsBuilder.fromHttpUrl(config.getAPIUrl(OpenAPIConstants.SCRM.CUSTOMER_DETAIL)).queryParams(queryParamMap);
        OpenAPIResponse queryResponse = restTemplate.getForObject(queryBuilder.toUriString(), OpenAPIResponse.class);
        assertResultAndLog(queryResponse);
    }

    private CustomerDTO buildNewCustomer(List<BasicCustomerFieldDefineDTO> fieldDefineDTOS) {
        String currMills = String.valueOf(System.currentTimeMillis());
        Date nowDate = new Date();

        CustomerDTO customerDTO = new CustomerDTO();
        customerDTO.setName("OpenAPIDemo" + nameDateFormat.format(nowDate));
        customerDTO.setPhone(currMills.substring(0, 11));
        customerDTO.setDesc("OpenAPIDemo for timemillis:" + currMills);
        customerDTO.setTag("OpenAPIDemo");
        customerDTO.setExternalId(currMills);

        if (fieldDefineDTOS == null || fieldDefineDTOS.isEmpty()) {
            return customerDTO;
        }

        List<CustomerDictDTO> customerDictDTOS = new ArrayList<>();
        customerDTO.setCustomerDictList(customerDictDTOS);
        for (BasicCustomerFieldDefineDTO fieldDefineDTO : fieldDefineDTOS) {

            if (fieldDefineDTO.getDefaultValue() == null) {
                continue;
            }

            CustomerDictDTO customerDictDTO = new CustomerDictDTO();
            customerDictDTO.setDictCode(fieldDefineDTO.getCode());

            String val = fieldDefineDTO.getDefaultValue();
            if (StringUtils.isBlank(val)) {
                switch (fieldDefineDTO.getType()) {
                    case 2:
                    case 3:
                    case 4:
                    case 5:
                        // 下拉框，默认取第一个选项值
                        BasicCustomerFieldMutliCheckbox mutliCheckbox = JSON.parseObject(fieldDefineDTO.getTypeDefine(), BasicCustomerFieldMutliCheckbox.class);
                        val = mutliCheckbox.getOptions().get(0).getKey();
                        break;
                    case 8:
                    case 9:
                        // 日期类型默认值
                        val = dateFormat.format(nowDate);
                        break;
                    case 11:
                    case 12:
                        // 手机号类型
                        val = customerDTO.getPhone();
                        break;
                    default:
                        val = "1";
                        break;
                }
            }
            customerDictDTO.setValue(val);
            customerDictDTOS.add(customerDictDTO);
        }

        return customerDTO;
    }


}
