package com.dz.openapi.demo.sms;


import com.dz.openapi.core.OpenAPIResponse;
import com.dz.openapi.demo.BaseWebTest;
import com.dz.openapi.demo.OpenAPIConstants;
import com.dz.openapi.demo.sms.model.SmsFormManageDisPO;
import com.dz.openapi.demo.sms.model.SmsSendOpenapiData;
import com.dz.openapi.demo.sms.model.SmsTemplateResponse;
import com.dz.openapi.demo.sms.util.SmsTemplateUtils;

import org.junit.jupiter.api.Test;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.util.UriComponentsBuilder;

import java.util.ArrayList;
import java.util.List;

import lombok.extern.slf4j.Slf4j;

@Slf4j
public class SMSTest extends BaseWebTest {

    @Test
    public void smsTest() {

        // 查询模板
        MultiValueMap<String, String> queryParamMap = new LinkedMultiValueMap<>();
        queryParamMap.add("templateName", "");
        UriComponentsBuilder queryBuilder = UriComponentsBuilder.fromHttpUrl(config.getAPIUrl(OpenAPIConstants.SMS.TEMPLATE_LIST)).queryParams(queryParamMap);
        SmsTemplateResponse queryResponse = restTemplate.getForObject(queryBuilder.toUriString(), SmsTemplateResponse.class);
        assertResultAndLog(queryResponse);

        if (queryResponse.getData() == null || queryResponse.getData().isEmpty()) {
            log.warn("租户下无短信模板配置，短信发送无法执行");
            return;
        }

        // 生成短信发送对象
        SmsFormManageDisPO formManageDisPO = queryResponse.getData().get(0);
        SmsSendOpenapiData smsSendOpenapiData = buildSmsSendOpenapiData(formManageDisPO);

        // 发送短信
        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        HttpEntity<SmsSendOpenapiData> sendRequest = new HttpEntity<>(smsSendOpenapiData, headers);
        OpenAPIResponse response = restTemplate.exchange(
                config.getAPIUrl(OpenAPIConstants.SMS.SEND), HttpMethod.POST, sendRequest, OpenAPIResponse.class).getBody();
        assertResultAndLog(response);
    }

    private SmsSendOpenapiData buildSmsSendOpenapiData(SmsFormManageDisPO formManageDisPO) {
        // 生成短信发送对象
        SmsSendOpenapiData sendOpenapiData = new SmsSendOpenapiData();
        sendOpenapiData.setTemplateId(String.valueOf(formManageDisPO.getId()));

        List<SmsSendOpenapiData.PhoneParam> phoneParamList = new ArrayList<>();
        sendOpenapiData.setPhoneParam(phoneParamList);

        SmsSendOpenapiData.PhoneParam phoneParam = new SmsSendOpenapiData.PhoneParam();
        phoneParam.setPhone("100000000000");
        List<String> paramDefs = SmsTemplateUtils.getContentParams(formManageDisPO.getFormContent());
        if (paramDefs.size() > 0) {
            List<String> paramDatas = new ArrayList<>();
            for (int i = 0; i < paramDefs.size(); i++) {
                paramDatas.add("变量" + (i + 1));
            }
            phoneParam.setParamDatas(paramDatas);
        }
        phoneParamList.add(phoneParam);

        return sendOpenapiData;
    }
}
